#region LICENSE
// Copyright (C) 2003 Ecmel Ercan
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Ecmel Ercan (ecmel@ercansoy.com)
#endregion

using System;
using Gtk;
using Glade;

public class GladeDialog
{
    protected ResponseHandler response_handler;
    protected Glade.XML gxml;
    [Glade.Widget] protected Dialog glade_dialog;

    protected GladeDialog (string resource, Window parent)
    {
        gxml = new Glade.XML (null, resource, "glade_dialog", null);
        gxml.Autoconnect (this);
        response_handler = new ResponseHandler (DialogResponse);

        if (parent != null)
        {
            glade_dialog.TransientFor = parent;
            glade_dialog.SetPosition(WindowPosition.CenterOnParent);
        }
    }

    public void Show ()
    {
        glade_dialog.Show ();
    }

    public void Present ()
    {
        glade_dialog.Present ();
    }

    public void Hide ()
    {
        glade_dialog.Hide ();
    }

    public void Destroy ()
    {
        glade_dialog.Destroy ();
    }

    public Dialog Self
    {
        get
        {
            return glade_dialog;
        }
    }

    public event ResponseHandler Response
    {
        add
        {
            lock (this)
            {
                glade_dialog.Response += response_handler;
                response_event += value;
            }
        }
        remove
        {
            lock (this)
            {
                glade_dialog.Response -= response_handler;
                response_event -= value;
            }
        }
    }

    public int Run ()
    {
        return glade_dialog.Run ();
    }

    private event ResponseHandler response_event = null;

    private void DialogResponse (object obj, ResponseArgs args)
    {
        response_event (this, args);
    }

}
